<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace MapModule\Lib;


use App\Lib\PluginExportTasks;
use Cake\Core\Configure;
use Cake\Log\Log;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use itnovum\openITCOCKPIT\Core\FileDebugger;
use MapModule\Model\Table\MapsTable;
use MapModule\Model\Table\RotationsTable;

class ExportTasks implements PluginExportTasks {

    public function __construct() {
        Configure::load('nagios');
        $this->conf = Configure::read('nagios.export');
    }

    public function beforeExport(): bool {
        if (!\Cake\Core\Plugin::isLoaded('DistributeModule')) {
            return true;
        }

        //Create zip file with all maps per satellite if DistributeModule is loaded

        /** @var MapsTable $MapsTable */
        $MapsTable = TableRegistry::getTableLocator()->get('MapModule.Maps');
        /** @var RotationsTable $RotationsTable */
        $RotationsTable = TableRegistry::getTableLocator()->get('MapModule.Rotations');
        /** @var \DistributeModule\Model\Table\SatellitesTable $SatellitesTable */
        $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');

        foreach ($SatellitesTable->getSatellitesAsList() as $satelliteId => $satelliteName) {
            $json = [
                'maps'      => [],
                'rotations' => []
            ];
            $files = [];

            $maps = $MapsTable->getMapsBySatelliteId($satelliteId, false);
            foreach ($maps as $map) {
                $mapForSatellite = $MapsTable->getMapForSatelliteExport($map['id'], $satelliteId);
                $json['maps'][] = $mapForSatellite;
                $files = Hash::merge($files, $this->getFilesForZipArchive($mapForSatellite)); //Merge all files of all maps on this satellite into one zip
            }

            $json['rotations'] = $RotationsTable->getRotationsWithMapsBySatelliteId($satelliteId);

            if (!is_dir($this->conf['satellite_path'] . $satelliteId)) {
                mkdir($this->conf['satellite_path'] . $satelliteId);
            }

            $mapZipArchive = $this->conf['satellite_path'] . $satelliteId . DS . 'maps.zip';
            $mapJson = $this->conf['satellite_path'] . $satelliteId . DS . 'maps.json';
            if (file_exists($mapZipArchive)) {
                unlink($mapZipArchive);
            }

            if (!empty($files)) {
                //Cannot create an empty zip
                $zipArchive = new \ZipArchive();
                if ($zipArchive->open($mapZipArchive, \ZipArchive::CREATE) !== true) {
                    Log::error('Cant create zip file');
                } else {
                    $directories = [];

                    foreach ($files as $filename => $file) {
                        $path = pathinfo($filename)['dirname'];

                        // Does directory already exists in zip?
                        if (!isset($directories[$path])) {
                            $zipArchive->addEmptyDir($path . DS);
                            $directories[$path] = true;
                        }

                        $zipArchive->addFile($file, $filename);
                    }
                    $zipArchive->close();
                }

            }

            if (file_exists($mapJson)) {
                unlink($mapJson);
            }

            $fd = fopen($mapJson, 'w+');
            fwrite($fd, json_encode($json, JSON_PRETTY_PRINT));
            fclose($fd);

        }

        return true;
    }

    public function afterExport(): bool {
        return true;
    }

    /**
     * @param array $mapForSatellite
     * @return array
     */
    private function getFilesForZipArchive($mapForSatellite) {
        $basePath = APP . '../' . 'plugins' . DS . 'MapModule' . DS . 'webroot' . DS . 'img' . DS;

        $files = [];

        if (!empty($mapForSatellite['background'])) {
            if (file_exists($basePath . 'backgrounds' . DS . $mapForSatellite['background'])) {
                $files['backgrounds' . DS . $mapForSatellite['background']] = $basePath . 'backgrounds' . DS . $mapForSatellite['background'];
            }
        }

        //Add stateless icons
        foreach ($mapForSatellite['mapicons'] as $mapicon) {
            if (file_exists($basePath . 'icons' . DS . $mapicon['icon'])) {
                $files['icons' . DS . $mapicon['icon']] = $basePath . 'icons' . DS . $mapicon['icon'];
            }
        }

        //Add missing icon
        $files['items/missing.png'] = $basePath . 'items' . DS . 'missing.png';

        //Add iconsets
        $iconsets = array_unique(Hash::extract($mapForSatellite, 'mapitems.{n}.iconset'));
        foreach ($iconsets as $iconset) {
            if (is_dir($basePath . 'items' . DS . $iconset)) {
                // ITC-2906 Add all icons with pull path to the zip archive - not just the parent directory
                foreach (scandir($basePath . 'items' . DS . $iconset) as $iconsetImageName) {
                    if ($iconsetImageName === '.' || $iconsetImageName === '..') {
                        continue;
                    }
                    $files['items' . DS . $iconset . DS . $iconsetImageName] = $basePath . 'items' . DS . $iconset . DS . $iconsetImageName;
                }

            }
        }

        return $files;
    }
}
